#!/usr/bin/env ruby 
#encoding: UTF-8

=begin
/******************************************************************************
 *   Copyright © 2017-2017, Michael Uplawski <michael.uplawski@uplawski.eu>   *
 *                                                                            *
 *   This program is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 3 of the License, or        *
 *   (at your option) any later version.                                      *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   You should have received a copy of the GNU General Public License        *
 *   along with this program; if not, write to the                            *
 *   Free Software Foundation, Inc.,                                          *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                *
 ******************************************************************************/
=end

require_relative 'constants'
require_relative 'logging'

class Adder
  #@@events_file = File.dirname(__FILE__) << File::Separator << 'events.rb'
  #@@events_backup = File.dirname(__FILE__) << File::Separator << 'bak_events.rb'
  self::extend(Logging)
  @@log = self::init_logger
  # Do not present as 'Adder' on the command-line
  self::log_label=$APPNAME

  def self::add(options) 
    @log.debug('options are ' << options.to_s)
    if options.file
      @@events_file = options.file
      @@events_backup = File.dirname(@@events_file) << File::Separator << 'bak_' << File.basename(@@events_file)
    else
      @log.error('To add events, you must name a file on the command-line. Aborting')
      exit false
    end
    if(!options.event || !options.year)
      @log.error("To add events, you must provide them on the command-line.\n\tPSE start the program with option -h or --help to see an option-overview.")
      exit false
    end

    @@log.level = $LOG_LEVEL if $LOG_LEVEL 
    fields = [:event, :year, :background]
    @log.debug('options are ' << options.to_s)
    if options[:event] && options[:year]
      if !options[:background]
        @log.warn('Optional background information is missing')
        bg = ''
      else
        bg = options.background
      end

      ev_def = "$events << [\"" << options.event << "\", " << options.year << ", \"" << bg << "\"]" 
      @log.debug('writing events-backup to ' << @@events_backup)
      File.open(@@events_backup, 'w+') {|bak| bak.write( File.read(@@events_file))}
      @log.debug('adding new event to the list: ' << ev_def)
      File.open(@@events_file, 'a') {|evf| evf.puts(ev_def)}
      @log.info('A new event has been added to ' << @@events_file << '.')
    else
      opts = options.to_h.keys
      @log.error('Option missing: ' << fields.each {|f| opts.delete(f) }.join(', '))
      exit false
    end
  end
end

