#encoding: UTF-8

=begin
/******************************************************************************
 *   Copyright © 2014-2017, Michael Uplawski <michael.uplawski@uplawski.eu>   *
 *                                                                            *
 *   This program is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 3 of the License, or        *
 *   (at your option) any later version.                                      *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   You should have received a copy of the GNU General Public License        *
 *   along with this program; if not, write to the                            *
 *   Free Software Foundation, Inc.,                                          *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                *
 ******************************************************************************/
=end

# BusyIndicator provides a way to show 'activity' in a terminal-window, while
# the main thread does something else. For a usage example see the test-code at
# the bottom of this file and execute the file (e.g. ruby ./busy_indicator.rb)
class BusyIndicator
  attr_writer :width
  def initialize(start = true, width = nil)
    # defaults
    @width = width && width >= 3 ? width : 3
    @sequence = %w"OOO ooo ___ ooo"
    run if start
  end

  # change sequence for the following run().
  def sequence=(seq)
    @sequence = seq
    @width = seq[0].length
  end

  # start busy-indicator with the current settings for width and sequence.
  def run() 
    @thr = busy_indicator(@width) if !@thr || !@thr.alive?
  end

  # stop busy-indicator and show comment-string if given.
  def stop(comment = nil)
    @thr.terminate
    @thr.join(0.1)
    print ("\b" * @width)
    print ("%+#{@width}s\n" %comment) if comment
  end

  # returns true or false, so that !stopped? == running?.
  def running?
    @thr.alive?
  end

  # returns true or false, so that !stopped? == running?.
  def stopped?
    !running?
  end

  private
  
  # does it.
  def busy_indicator(width)
    tobj = Thread.new() do
      loop do 
        @sequence.each do |s|
          print "%+#{width}s" %s
          sleep 0.1
          print ("\b" * width)
        end
      end
    end
  end
end

# ========== TEST =============
if($0 == __FILE__)
  width = 20
  2.times do |i|
    puts "I am busy, pse wait..."
    thr = BusyIndicator.new(true, width)
    sleep 3
    # effect of comment != nil and comment == nil
    thr.stop("Okay") if i == 0
    thr.stop() if i != 0
  end
  bi = BusyIndicator.new(false)
  # change sequence
  bi.sequence = %w"E [ ( | ¦ ; , . o O D P F"
  bi.run
  sleep 3
  bi.stop("Stopped")
  # change width
  bi.width = 8
  bi.run
  sleep 3
  bi.stop("stopped again")
end

