#encoding: UTF-8

=begin
/******************************************************************************
 *   Copyright © 2014-2017, Michael Uplawski <michael.uplawski@uplawski.eu>   *
 *                                                                            *
 *   This program is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 3 of the License, or        *
 *   (at your option) any later version.                                      *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   You should have received a copy of the GNU General Public License        *
 *   along with this program; if not, write to the                            *
 *   Free Software Foundation, Inc.,                                          *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                *
 ******************************************************************************/
=end

# Functions to apply colors to terminal output.
# This is stolen from the Internet and I have lost track of my own additions
# and modifications.

COLORS = {:default => 9, :black => 0, :red => 1, :green => 2, :yellow => 3, :blue => 4, :purple => 5, :cyan => 6, :white => 7 }

BG = 4
FG = 3
REGULAR = 0
BOLD = 1
UNDERLINE = 4
BLINK = 5
SWAP = 7
NEUTRAL = 0

STYLES = {:regular => REGULAR, :bold => BOLD, :underline => UNDERLINE, :blink => BLINK, :swap => SWAP, :neutral => NEUTRAL}

# Colorizes the given text. Color-code is either an escape-sequence or one of
# the symbols representing color-names in the COLORS hash.
def colorize(text, color_code)
  if (COLORS.keys.include?(color_code) )
    "\033[3#{COLORS[color_code]}m#{text}\033[0m"
  else
    "#{color_code}#{text}\033[0m"
  end
end

def style(text, style_code)
  "#{style_code}#{text}\033[0m"
end
# a function which allows to manipulate every known aspect of the ansi-output.
def colored_output(output_text, fg_color = :default, bg_color = :default, style = :regular , mode = :neutral )
  "\033[%i;%i;%i%i;%i%im%s\033[0m" %[STYLES[mode.to_sym], STYLES[style.to_sym], FG, COLORS[fg_color.to_sym], BG, COLORS[bg_color.to_sym], output_text]
end

# convenience functions
def red(text); colorize(text, "\033[31m"); end
def green(text); colorize(text, "\033[32m"); end
def yellow(text); colorize(text, "\033[33m"); end
def purple(text); colorize(text, "\033[35m"); end
def cyan(text); colorize(text, "\033[36m"); end
def blue(text); colorize(text, "\033[34m"); end
def white(text); colorize(text, "\033[37m"); end

def black_on_white(text); colorize(colorize(text, "\033[30m"), "\033[47m");end
def white_on_black(text); colorize(colorize(text, "\033[37m"), "\033[40m");end

def bold(text); style(text, "\033[01m");end
def underline(text); style(text, "\033[04m");end

