#encoding: UTF-8

=begin
/******************************************************************************
 *   Copyright © 2017-2017, Michael Uplawski <michael.uplawski@uplawski.eu>   *
 *                                                                            *
 *   This program is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 3 of the License, or        *
 *   (at your option) any later version.                                      *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   You should have received a copy of the GNU General Public License        *
 *   along with this program; if not, write to the                            *
 *   Free Software Foundation, Inc.,                                          *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                *
 ******************************************************************************/
=end

require_relative 'color_output'

# Functions which extend the String-class. Simply require this file to add
# capabilities to each String-object in your code. 

# For historical reasons. Probably obsolete
if(! String.method_defined?(:end_with?))
  class String
    def end_with?(str)
      isso = false;
      if(str.respond_to?(:to_str) && self.include?(str))
        isso = (slice(rindex(str), (size()-1)) == str)
      end
      return isso
    end
  end
end

# Not obsolete

class String

  # Draws a box around the content.
  # TODO: the :padding option is currently disabled as it confuses me.
  def box(opt = {})
    # ---------------
    # Welcome to hell
    # ---------------

    color = opt[:color]

    upleft = color ? colorize("\u250F", color) : "\u250F"
    upright = color ? colorize("\u2513", color) : "\u2513"
    downleft = color ? colorize("\u2517", color) : "\u2517"
    downright = color ? colorize("\u251B", color) : "\u251B"

    vert = color ? colorize("\u2503", color) : "\u2503"
    hor = color ? colorize("\u2501", color) : "\u2501"

    left = opt[:left]
    top = opt[:top]
    # padding = opt[:padding]

    min_width = opt[:min_width]

    size = `stty size`.split.map { |x| x.to_i }.reverse
    left ||= 0
    top ||= 0
    # padding ||= 0
    
    min_width ||= 15
    
    # max_width is the width of the screen - left margin - 2x border
    max_width = size[0] - left - 2

    gsub!("\t", '    ')
    width = max_width
    # wrap lines, maybe
    wrap(width , /^\d+\)/)
    # determine the width of the box 
    width = self.split("\n").max {|l1, l2| l1.chomp.length <=> l2.chomp.length}.length  
    width = [width, min_width].max

    box = ''
    # top margin
    box << "\n" * top
    # top border, left margin
    box << ' ' * left << upleft.dup << hor.dup * width   << upright.dup << "\n"
    # top padding, left margin and so on ...
    box << ' ' * left << vert.dup << ' ' * width << vert.dup << "\n"
    # lines
    self.split("\n").each do |line|
      if(line && !line.empty?)
        nl = ' ' * left << vert.dup << "%-#{width}s" %line
        while(nl.length < width )
          nl << ' '
        end
        nl << vert.dup << "\n"
      else
        nl = ' ' * left << vert.dup << ' ' * width  << vert.dup << "\n"
      end
      box << nl
    end
    # bottom padding
    box << ' ' * left << vert.dup << ' ' * width << vert.dup << "\n"
    # bottom border
    box << ' ' * left << downleft.dup << hor.dup * width << downright.dup
    return box
  end

  def box!(opt = {})
    self.replace(String.new(box(opt)))
  end

  # A wrap-function which works here, but may fail everywhere else.
  def wrap(width, delimiter = "\n")
    # YEAH. Ruby is crap because its regular expressions do not work as they
    # should with special characters. And your coffee mug has a speech
    # impediment. Face it! 
    words = self.scan(/\p{P}*[\wöÖüÜäÄßéèàôûùîçÇœæŒÆ]+\p{P}*/)
   
    lines = []
    lines << ''
    li = 0
    words.each do |w|
      w.strip!
      if( w.length + lines[li].length + 1 > width || w != words.first && w.match(delimiter) )
        lines[li].squeeze!(' ')
        lines << ''
        li += 1
      else
      end
      lines[li] << w 
      lines[li] << ' ' if w != words.last
    end
    self.replace(lines.join("\n"))
  end

end

#--------------- TEST ---------------
if __FILE__ == $0
  s = "Ein König mit Krone ist besser als ohne,\n\n(du Nase)!!".wrap(12)
  puts s

  s = "1) Ein Koenig mit Krone ist besser als ohne,\n\n(du Nase)!!\n2) We are not in Kansas anymore.\n3) Toto? Ich glaub, der Köter hieß Toto... ".wrap(12, /^\d/)
  puts s
  

  # puts "Ein Koenig mit Krone ist besser als ohne,\n\ndu Nase!!".box(:padding=>2)

  s = "1) Didudel Doedel didödeledel dödeldödel dödeldödel du dödel. DOEDEL !\n"
  s << "2) Man gab uns mancherlei auf Erden.\nZum Denken gab man uns die Stirn."
  s << "2a) Man gab uns Herz- und Leibbeschwerden.\nDoch auch den Himmel und den Zwirn."
  s << "3) Himmel, Ding und Zwirn."
  s.wrap(12, /\d\)/)
  puts s.box(:min_width=>20, :left => 20)
end
