#encoding: UTF-8

=begin
/***************************************************************************
 *   Copyright © 2017-2017, Michael Uplawski <michael.uplawski@uplawski.eu>*
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require 'io/wait'
require 'io/console'

# Provides functions to convert terminal input to desired data-types and does
# some verifications.
# Keyboard-input is always non-blocking, to allow syntax-checking on the fly
# and to accelerate the work-flow, wherever possible.
class UserInput
  # Brutal log-function: See your desired message, then hit 'q' to exit program.
  def self::debout(str)
    if $DEBUG == :DEBUG
      puts str
      if(wait_for_user().chr == 'q')
        exit true
      end
    end
  end

  def self::string_input
    inp = ""
    loop do
      begin
        char = wait_for_user()
        if char == 10 || char == 13
          return inp
        elsif( 'q' == char.chr.downcase)
          exit true
        else
          inp << char
        end
      rescue Interrupt
        exit true
      end
    end
  end

  def self::int_input(range = nil)
    ar = []
    loop do
      begin
        char = wait_for_user()
        if char == 10 || char == 13
          if(range && range.min > ar.join.to_i)
            puts "\n%d is too small. Choose a number from the list of events or 0 (zero)" %ar.join.to_i
            ar.clear
          else
            return ar.join.to_i
          end
        elsif( 'q' == char.chr.downcase)
          print "\n\tBye "
          return :quit_game
        else
          num = char - 48
          ar << num
          print num
          if(range && range.max < ar.join.to_i )
            puts "\n%d is becoming too big. Choose a number from the list of events or 0 (zero)" %ar.join.to_i
            ar.clear
          end
        end
      rescue Interrupt
        exit true
      end
    end
  end

  def self::float_input
    ar = []
    loop do
      begin
        char = wait_for_user()
        if char == 10 || char == 13
          return ar.join.to_f
        elsif( 'q' == char.chr.downcase)
          print "\n\tBye "
          return :quit_game
        else
          ar << char
        end
      rescue Interrupt
        exit true
      end
    end
  end

  def self::char_input
    begin
      char = wait_for_user()
      if( 'q' == char.chr.downcase)
        print "\n\tBye "
        return :quit_game
      elsif ('a'..'z').member?(char.chr.downcase)
        return char
      end
    rescue Interrupt
      exit true
    end
  end


  def self::input
    inp = []
    loop do
      begin
        char = wait_for_user()
        if char == 10 || char == 13
          return inp
        elsif( 'q' == char.chr.downcase)
          print "\n\tBye. "
          return :quit_game
        elsif ('a'..'z').member?(char.chr.downcase)
          return char.chr
        else
          inp << char - 48
        end
      rescue Interrupt
        exit true
      end

    end
  end

  def self::int_array_input(len, range = nil)
    ar = []
    until ar.length == len
      if(range)
        print ' (Available: '
        disp = range.to_a - ar
        if disp.length > 2
          puts disp.join(', ') << ')' 
        elsif disp.length == 2
          puts disp.first.to_s << ' or ' << disp.last.to_s << ')'
        else
          puts disp.first.to_s << ')'
        end
      end

      begin
        char = wait_for_user()
        if( 'q' == char.chr.downcase)
          print "\n\tBye. "
          return :quit_game
        elsif range && range.member?(char - 48)
          num = char - 48
          if(ar.member?(num)) 
            puts '%d cannot happen twice!' %num
            puts 'Try again or push "q" to quit.'
          else
            ar << num
          end
        else
          print "%s is not a valid input." %char.chr
          if(range)
            print ' (Available: '
            disp = range.to_a - ar
            if disp.length > 2
              puts disp.join(', ') << ')' 
            elsif disp.length == 2
              puts disp.first.to_s << ' or ' << disp.last.to_s << ')'
            else
              puts disp.first.to_s << ')'
            end
          end
          puts 'Try again or push "q" to quit.'
        end
      rescue Interrupt
        exit true
      end
    end
    return ar
  end

  # For cases when no data-type can be imposed for the input, but ultimately an
  # integer is needed. [4,3,2] becomes 432.
  # Beware, that the character-values must have been converted to int, already,
  # by substracting 48 from any number-symbol, or by converting its character
  # incarnation to int.
  def self::a2i(ar)
    if ar.respond_to?(:to_ary)
      ar.reverse!
      res = 0
      ar.each_with_index do |n, i|
        res += (n * 10**i)
      end
      return res
    else
      puts "Invalid input " << ar.to_s
      return 0
    end
  end

  private

  # The marvellous and mystic, heard-about so-long... unblocking read from STDIN!!
  def self::wait_for_user()
    char = nil
    # There is a difference which makes me prefer the below code from STDIN.raw(args)
    # You can try
    # char = STDIN.raw(&:getc)	
    # .., which does *not* work the same way and it won't for me.

    STDIN.raw do 
      STDIN.noecho do
        until (STDIN.ready?)
          sleep(0.1)
        end
        char = (STDIN.read_nonblock(1).ord rescue nil)
      end
    end
    return char
  end


end
# -------- TEST
if __FILE__ == $0
  str = UserInput::input
  puts "You say: " << str.to_s
  puts "As Integer: " << UserInput::a2i(str).to_s
  print 'an integer from 23 to 45: '
  int = UserInput::int_input(23..45)
  puts " You say " << int.to_s
end
